#version 300 es

precision highp float;

const float c_PI = 3.14159;

uniform float u_Time;
uniform vec2 u_Offset;
uniform float u_Speed;
uniform float u_Scale; // масштабирование градиента [0..1) - отдаление, (1..] - приближение
uniform int u_Complexity; // 9

uniform float u_HUEOffset;
uniform float u_SatFactor;
uniform float u_LightFactor;

uniform sampler2D u_Mask;
uniform int u_UseMask;

const float color_intensity = 0.8;

in vec2 v_GCoord;

layout(location = 0) out vec4 o_FragColor;

vec3 rgb2hsv(vec3 c) {
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 hsv2rgb(vec3 c) {
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

void main() {
    vec2 p = 2.0 * v_GCoord / u_Scale;

    p += u_Offset;

    float t = u_Time / u_Speed;

    for(int i = 1; i < u_Complexity; i++) {
        vec2 newp = p;
        float ii = float(i);
        newp.x += 0.55 / ii * sin(ii * c_PI * p.y + t * 0.01 + cos((t / (10.0 * ii)) *ii));
        newp.y += 0.55 / ii * cos(ii * c_PI * p.x + t * 0.01 + sin((t / (10.0 * ii)) * ii));
        p = newp;
    }

    vec3 col = vec3(cos(p.x + p.y + 3.0 + t) * 0.5 + 0.5
                    , sin(p.x + p.y + 6.0 + t) * 0.5 + 0.5
                    , (sin(p.x + p.y + 9.0 + t) + cos(p.x + p.y + 12.0 + t)) * 0.25 + 0.5);

    col = rgb2hsv(col);
    col.x = mod(col.x + u_HUEOffset, 1.0);
    col.y = col.y * u_SatFactor;
    col.z = col.z * u_LightFactor;
    col = hsv2rgb(col);

    float a = 1.0;

    if (u_UseMask == 1) {
        vec2 texCoords = vec2(v_GCoord.x + 0.5, 1.0 - (v_GCoord.y + 0.5));
        a = texture(u_Mask, texCoords).r;
    }

    o_FragColor = vec4(col, 1.0) * a;
}