#version 300 es
#extension GL_OES_standard_derivatives : enable

precision highp float;

uniform float u_Time;
uniform vec2 u_Offset;
uniform float u_Speed;
uniform float u_Scale; // масштабирование градиента [0..1) - отдаление, (1..] - приближение
uniform int u_Complexity;

uniform float u_HUEOffset;
uniform float u_SatFactor;
uniform float u_LightFactor;

uniform sampler2D u_Mask;
uniform int u_UseMask;

in vec2 v_GCoord;

layout(location = 0) out vec4 o_FragColor;

float map(float value, float inMin, float inMax, float outMin, float outMax) {
    return (value - inMin) * (outMax - outMin) / (inMax - inMin) + outMin;
}

vec2 map(vec2 v, float inMin, float inMax, float outMin, float outMax) {
    float x = map(v.x, inMin, inMax, outMin, outMax);
    float y = map(v.y, inMin, inMax, outMin, outMax);

    return vec2(x, y);
}

vec3 rgb2hsv(vec3 c) {
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 hsv2rgb(vec3 c) {
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

void main() {
    vec2 p = map(v_GCoord, -1.0, 1.0, 0.0, 1.0);

    p.x += u_Offset.x;
    p.y += u_Offset.y;

    p.x = (p.x * 0.25 + 2.0) / u_Scale;
    p.y = (p.y * 0.25 + 2.0) / u_Scale;

    vec3 col;
    float t = u_Time / u_Speed;

    for(float j = 0.0; j < float(u_Complexity); j++) {
        for(float i = 1.0; i < float(u_Complexity) * 4.0; i++) {
            p.x += 0.1 / (i + j) * sin(i * 9.0 * p.y + t * 0.19 + cos((0.4335 * t / (7.11 * i)) * i - j));
            p.y += 0.1 / (i + j) * cos(i * 7.34 * p.x + t * 0.32 + sin((t * 0.7713 / (6.0 * i)) * i + j));
        }

        col[int(j)] = (cos(7.0 * p.x * p.x) + sin(7.0 * p.y * p.y)) ;
    }

    col = rgb2hsv(vec3(0.25 * col -0.3 + sin(0.75 + normalize(col + 10.5))));
    col.x = mod(col.x + u_HUEOffset, 1.0);
    col.y = col.y * u_SatFactor;
    col.z = col.z * u_LightFactor;
    col = hsv2rgb(col);

    float a = 1.0;

    if (u_UseMask == 1) {
        vec2 texCoords = vec2(v_GCoord.x + 0.5, 1.0 - (v_GCoord.y + 0.5));
        a = texture(u_Mask, texCoords).r;
    }

    o_FragColor = vec4(col, 1.0) * a;
}