#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Offset;
uniform float u_Speed;
uniform float u_Scale; // масштабирование градиента [0..1) - отдаление, (1..] - приближение
uniform int u_Complexity;

uniform float u_HUEOffset;
uniform float u_SatFactor;
uniform float u_LightFactor;

uniform sampler2D u_Mask;
uniform int u_UseMask;

in vec2 v_GCoord;

layout(location = 0) out vec4 o_FragColor;

float hash(float x, float y)
{
    return fract(abs(sin(sin(123.321 + x) * (y + 321.123)) * 456.654));
}
float lerp(float a, float b, float t)
{
    return a * (1.0 - t) + b * t;
}
float perlin(float x, float y)
{
    float col = 0.0;
    for (int i = 0; i < 8; i++)
    {
        float fx = floor(x);
        float fy = floor(y);
        float cx = ceil(x);
        float cy = ceil(y);
        float a = hash(fx, fy);
        float b = hash(fx, cy);
        float c = hash(cx, fy);
        float d = hash(cx, cy);
        col += lerp(lerp(a, b, fract(y)), lerp(c, d, fract(y)), fract(x));
        col /= 2.0;
        x /= 2.0;
        y /= 2.0;
    }
    return col;
}
float dperlin(float x, float y)
{
    float d = perlin(x, y) * 800.0;
    return perlin(x + d, y + d);
}
float ddperlin(float x, float y)
{
    float d = perlin(x, y) * 800.0;
    return dperlin(x + d, y + d);
}

vec3 rgb2hsv(vec3 c) {
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 hsv2rgb(vec3 c) {
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

void main() {

    vec2 p = v_GCoord / u_Scale;

    p += u_Offset;

    p *= 800.0;
    float d = perlin(p.x, p.y) * 800.0 + sin(u_Time / (u_Speed * 100.)) * 800.;
    float u = p.x + d;
    float v = p.y + d;
    d = perlin(u, v) * 800.0;

    float color = perlin(p.x + d, p.y + d);

    vec3 col = vec3(2.0 * color) - vec3(color * .1, color * .2 - sin(u / 30.0) * .1, color * .3 + sin(v / 40.0) * .2);

    col = rgb2hsv(col);
    col.x = mod(col.x + u_HUEOffset, 1.0);
    col.y = col.y * u_SatFactor;
    col.z = col.z * u_LightFactor;
    col = hsv2rgb(col);

    float a = 1.0;

    if (u_UseMask == 1) {
        vec2 texCoords = vec2(v_GCoord.x + 0.5, 1.0 - (v_GCoord.y + 0.5));
        a = texture(u_Mask, texCoords).r;
    }

    o_FragColor = vec4(col, 1.0) * a;
}
